@@ -29,15 +29,21 @@
 
 from dumper import *
 
+def qform__std__array():
+    return arrayForms()
+
 def qdump__std__array(d, value):
     size = d.numericTemplateArgument(value.type, 1)
     d.putItemCount(size)
     d.putNumChild(size)
     if d.isExpanded():
         innerType = d.templateArgument(value.type, 0)
-        d.putArrayData(innerType, d.addressOf(value), size)
+        d.putPlotData(innerType, d.addressOf(value), size)
 
 
+def qform__std____1__array():
+    return arrayForms()
+
 def qdump__std____1__array(d, value):
     qdump__std__array(d, value)
 
@@ -55,6 +61,10 @@ def qdump__std__complex(d, value):
 
 
 def qdump__std__deque(d, value):
+    if d.isQnxTarget():
+        qdump__std__deque__QNX(d, value)
+        return
+
     innerType = d.templateArgument(value.type, 0)
     innerSize = innerType.sizeof
     bufsize = 1
@@ -87,19 +97,55 @@ def qdump__std__deque(d, value):
                     plast = pfirst + bufsize
                     pcur = pfirst
 
+def qdump__std__deque__QNX(d, value):
+    innerType = d.templateArgument(value.type, 0)
+    innerSize = innerType.sizeof
+    if innerSize <= 1:
+        bufsize = 16
+    elif innerSize <= 2:
+        bufsize = 8
+    elif innerSize <= 4:
+        bufsize = 4
+    elif innerSize <= 8:
+        bufsize = 2
+    else:
+        bufsize = 1
+
+    myoff = value['_Myoff']
+    mysize = value['_Mysize']
+    mapsize = value['_Mapsize']
+
+    d.check(0 <= mapsize and mapsize <= 1000 * 1000 * 1000)
+    d.putItemCount(mysize)
+    d.putNumChild(mysize)
+    if d.isExpanded():
+        with Children(d, mysize, maxNumChild=2000, childType=innerType):
+            map = value['_Map']
+            for i in d.childRange():
+                block = myoff / bufsize
+                offset = myoff - (block * bufsize)
+                if mapsize <= block:
+                    block -= mapsize
+                d.putSubItem(i, map[block][offset])
+                myoff += 1;
+
 def qdump__std____debug__deque(d, value):
     qdump__std__deque(d, value)
 
 
 def qdump__std__list(d, value):
+    if d.isQnxTarget():
+        qdump__std__list__QNX(d, value)
+        return
+
     impl = value["_M_impl"]
     node = impl["_M_node"]
     head = d.addressOf(node)
     size = 0
-    pp = d.dereference(head)
+    pp = d.extractPointer(head)
     while head != pp and size <= 1001:
         size += 1
-        pp = d.dereference(pp)
+        pp = d.extractPointer(pp)
 
     d.putItemCount(size, 1000)
     d.putNumChild(size)
@@ -113,6 +159,21 @@ def qdump__std__list(d, value):
                 d.putSubItem(i, (p + 1).cast(innerPointer).dereference())
                 p = p["_M_next"]
 
+def qdump__std__list__QNX(d, value):
+    node = value["_Myhead"]
+    size = value["_Mysize"]
+
+    d.putItemCount(size, 1000)
+    d.putNumChild(size)
+
+    if d.isExpanded():
+        p = node["_Next"]
+        innerType = d.templateArgument(value.type, 0)
+        with Children(d, size, maxNumChild=1000, childType=innerType):
+            for i in d.childRange():
+                d.putSubItem(i, p['_Myval'])
+                p = p["_Next"]
+
 def qdump__std____debug__list(d, value):
     qdump__std__list(d, value)
 
@@ -120,12 +181,43 @@ def qform__std__map():
     return mapForms()
 
 def qdump__std__map(d, value):
+    if d.isQnxTarget():
+        qdump__std__map__QNX(d, value)
+        return
+
     impl = value["_M_t"]["_M_impl"]
     size = int(impl["_M_node_count"])
     d.check(0 <= size and size <= 100*1000*1000)
     d.putItemCount(size)
     d.putNumChild(size)
 
+    if d.isExpanded():
+        pairType = d.templateArgument(d.templateArgument(value.type, 3), 0)
+        pairPointer = pairType.pointer()
+        with PairedChildren(d, size, pairType=pairType, maxNumChild=1000):
+            node = impl["_M_header"]["_M_left"]
+            for i in d.childRange():
+                with SubItem(d, i):
+                    pair = (node + 1).cast(pairPointer).dereference()
+                    d.putPair(pair)
+                if d.isNull(node["_M_right"]):
+                    parent = node["_M_parent"]
+                    while node == parent["_M_right"]:
+                        node = parent
+                        parent = parent["_M_parent"]
+                    if node["_M_right"] != parent:
+                        node = parent
+                else:
+                    node = node["_M_right"]
+                    while not d.isNull(node["_M_left"]):
+                        node = node["_M_left"]
+
+def qdump__std__map__QNX(d, value):
+    size = value['_Mysize']
+    d.check(0 <= size and size <= 100*1000*1000)
+    d.putItemCount(size)
+    d.putNumChild(size)
+
     if d.isExpanded():
         keyType = d.templateArgument(value.type, 0)
         valueType = d.templateArgument(value.type, 1)
@@ -142,7 +234,9 @@ def qdump__std__map(d, value):
         innerType = pairType
         if isCompact:
             innerType = valueType
-        node = impl["_M_header"]["_M_left"]
+        head = value['_Myhead']
+        node = head['_Left']
+        nodeType = head.type
         childType = innerType
         if size == 0:
             childType = pairType
@@ -153,7 +247,7 @@ def qdump__std__map(d, value):
                 childType=childType, childNumChild=childNumChild):
             for i in d.childRange():
                 with SubItem(d, i):
-                    pair = (node + 1).cast(pairPointer).dereference()
+                    pair = node.cast(nodeType).dereference()['_Myval']
                     if isCompact:
                         d.putMapName(pair["first"])
                         d.putItem(pair["second"])
@@ -163,17 +257,17 @@ def qdump__std__map(d, value):
                             with Children(d, 2):
                                 d.putSubItem("first", pair["first"])
                                 d.putSubItem("second", pair["second"])
-                if d.isNull(node["_M_right"]):
-                    parent = node["_M_parent"]
-                    while node == parent["_M_right"]:
-                        node = parent
-                        parent = parent["_M_parent"]
-                    if node["_M_right"] != parent:
-                        node = parent
+                if not node['_Right']['_Isnil']:
+                    node = node['_Right']
+                    while not node['_Left']['_Isnil']:
+                        node = node['_Left']
                 else:
-                    node = node["_M_right"]
-                    while not d.isNull(node["_M_left"]):
-                        node = node["_M_left"]
+                    parent = node['_Parent']
+                    while node == parent['_Right']['_Isnil']:
+                        node = parent
+                        parent = parent['_Parent']
+                    if node['_Right'] != parent:
+                        node = parent
 
 def qdump__std____debug__map(d, value):
     qdump__std__map(d, value)
@@ -194,10 +288,11 @@ def stdTreeIteratorHelper(d, value):
         nodeType = d.lookupType(nodeTypeName)
         data = node.cast(nodeType)["_M_value_field"]
         with Children(d):
-            try:
-                d.putSubItem("first", data["first"])
+            first = d.childWithName(data, "first")
+            if first:
+                d.putSubItem("first", first)
                 d.putSubItem("second", data["second"])
-            except:
+            else:
                 d.putSubItem("value", data)
             with SubItem(d, "node"):
                 d.putNumChild(1)
@@ -236,6 +331,10 @@ def qdump__std____cxx1998__set(d, value):
     qdump__std__set(d, value)
 
 def qdump__std__set(d, value):
+    if d.isQnxTarget():
+        qdump__std__set__QNX(d, value)
+        return
+
     impl = value["_M_t"]["_M_impl"]
     size = int(impl["_M_node_count"])
     d.check(0 <= size and size <= 100*1000*1000)
@@ -259,9 +358,34 @@ def qdump__std__set(d, value):
                     while not d.isNull(node["_M_left"]):
                         node = node["_M_left"]
 
+def qdump__std__set__QNX(d, value):
+    size = value['_Mysize']
+    d.check(0 <= size and size <= 100*1000*1000)
+    d.putItemCount(size)
+    d.putNumChild(size)
+    if d.isExpanded():
+        valueType = d.templateArgument(value.type, 0)
+        head = value['_Myhead']
+        node = head['_Left']
+        nodeType = head.type
+        with Children(d, size, maxNumChild=1000, childType=valueType):
+            for i in d.childRange():
+                d.putSubItem(i, node.cast(nodeType).dereference()['_Myval'])
+                if not node['_Right']['_Isnil']:
+                    node = node['_Right']
+                    while not node['_Left']['_Isnil']:
+                        node = node['_Left']
+                else:
+                    parent = node['_Parent']
+                    while node == parent['_Right']['_Isnil']:
+                        node = parent
+                        parent = parent['_Parent']
+                    if node['_Right'] != parent:
+                        node = parent
 
 def qdump__std__stack(d, value):
-    qdump__std__deque(d, value["c"])
+    d.putItem(value["c"])
+    d.putType(str(value.type))
 
 def qdump__std____debug__stack(d, value):
     qdump__std__stack(d, value)
@@ -273,6 +397,10 @@ def qdump__std__string(d, value):
     qdump__std__stringHelper1(d, value, 1)
 
 def qdump__std__stringHelper1(d, value, charSize):
+    if d.isQnxTarget():
+        qdump__std__stringHelper1__QNX(d, value, charSize)
+        return
+
     data = value["_M_dataplus"]["_M_p"]
     # We can't lookup the std::string::_Rep type without crashing LLDB,
     # so hard-code assumption on member position
@@ -285,6 +413,20 @@ def qdump__std__stringHelper1(d, value, charSize):
     d.check(0 <= size and size <= alloc and alloc <= 100*1000*1000)
     qdump_stringHelper(d, sizePtr, size * charSize, charSize)
 
+def qdump__std__stringHelper1__QNX(d, value, charSize):
+    size = value['_Mysize']
+    alloc = value['_Myres']
+    _BUF_SIZE = 16 / charSize
+    if _BUF_SIZE <= alloc: #(_BUF_SIZE <= _Myres ? _Bx._Ptr : _Bx._Buf);
+        data = value['_Bx']['_Ptr']
+    else:
+        data = value['_Bx']['_Buf']
+    sizePtr = data.cast(d.charType().pointer())
+    refcount = int(sizePtr[-1])
+    d.check(refcount >= -1) # Can be -1 accoring to docs.
+    d.check(0 <= size and size <= alloc and alloc <= 100*1000*1000)
+    qdump_stringHelper(d, sizePtr, size * charSize, charSize)
+
 def qdump_stringHelper(d, data, size, charSize):
     cutoff = min(size, d.stringCutOff)
     mem = d.readMemory(data, cutoff)
@@ -310,21 +452,33 @@ def qdump_stringHelper(d, data, size, charSize):
 
 
 def qdump__std____1__string(d, value):
-    inner = d.childAt(d.childAt(value["__r_"]["__first_"], 0), 0)
-    size = int(inner["__size_"])
-    alloc = int(inner["__cap_"])
-    data = d.pointerValue(inner["__data_"])
+    base = d.addressOf(value)
+    firstByte = d.extractByte(base)
+    if firstByte & 1:
+        # Long/external.
+        data = d.extractPointer(base + 2 * d.ptrSize())
+        size = d.extractInt(base + d.ptrSize())
+    else:
+        # Short/internal.
+        size = firstByte / 2
+        data = base + 1
     qdump_stringHelper(d, data, size, 1)
     d.putType("std::string")
 
 
 def qdump__std____1__wstring(d, value):
-    inner = d.childAt(d.childAt(value["__r_"]["__first_"], 0), 0)
-    size = int(inner["__size_"]) * 4
-    alloc = int(inner["__cap_"])
-    data = d.pointerValue(inner["__data_"])
-    qdump_stringHelper(d, data, size, 4)
-    d.putType("std::wstring")
+    base = d.addressOf(value)
+    firstByte = d.extractByte(base)
+    if firstByte & 1:
+        # Long/external.
+        data = d.extractPointer(base + 2 * d.ptrSize())
+        size = d.extractInt(base + d.ptrSize())
+    else:
+        # Short/internal.
+        size = firstByte / 2
+        data = base + 4
+    qdump_stringHelper(d, data, size * 4, 4)
+    d.putType("std::xxwstring")
 
 
 def qdump__std__shared_ptr(d, value):
@@ -335,7 +489,7 @@ def qdump__std__shared_ptr(d, value):
         return
 
     if d.isSimpleType(d.templateArgument(value.type, 0)):
-        d.putValue("%s @0x%x" % (i.dereference(), d.pointerValue(i)))
+        d.putValue("%s @0x%x" % (d.simpleValue(i.dereference()), d.pointerValue(i)))
     else:
         i = d.expensiveDowncast(i)
         d.putValue("@0x%x" % d.pointerValue(i))
@@ -355,7 +509,7 @@ def qdump__std____1__shared_ptr(d, value):
         return
 
     if d.isSimpleType(d.templateArgument(value.type, 0)):
-        d.putValue("%s @0x%x" % (i.dereference().value, d.pointerValue(i)))
+        d.putValue("%s @0x%x" % (d.simpleValue(i.dereference()), d.pointerValue(i)))
     else:
         d.putValue("@0x%x" % d.pointerValue(i))
 
@@ -374,7 +528,7 @@ def qdump__std__unique_ptr(d, value):
         return
 
     if d.isSimpleType(d.templateArgument(value.type, 0)):
-        d.putValue("%s @0x%x" % (i.dereference(), d.pointerValue(i)))
+        d.putValue("%s @0x%x" % (d.simpleValue(i.dereference()), d.pointerValue(i)))
     else:
         i = d.expensiveDowncast(i)
         d.putValue("@0x%x" % d.pointerValue(i))
@@ -384,14 +538,15 @@ def qdump__std__unique_ptr(d, value):
         d.putSubItem("data", i)
 
 def qdump__std____1__unique_ptr(d, value):
-    i = d.childAt(d.childAt(value["__ptr_"], 0), 0)
+    #i = d.childAt(d.childAt(value["__ptr_"], 0), 0)
+    i = d.childAt(value["__ptr_"], 0)["__first_"]
     if d.isNull(i):
         d.putValue("(null)")
         d.putNumChild(0)
         return
 
     if d.isSimpleType(d.templateArgument(value.type, 0)):
-        d.putValue("%s @0x%x" % (i.dereference().value, d.pointerValue(i)))
+        d.putValue("%s @0x%x" % (d.simpleValue(i.dereference()), d.pointerValue(i)))
     else:
         d.putValue("@0x%x" % d.pointerValue(i))
 
@@ -445,12 +600,12 @@ def qdump__std__unordered_map(d, value):
                         d.putField("iname", d.currentIName)
                         d.putName("[%s] %s" % (i, pair["first"]))
                         d.putValue(pair["second"])
-                    p = d.dereference(p)
+                    p = d.extractPointer(p)
         else:
             with Children(d, size, childType=pairType):
                 for i in d.childRange():
                     d.putSubItem(i, d.createValue(p + ptrSize - offset, pairType))
-                    p = d.dereference(p + offset)
+                    p = d.extractPointer(p + offset)
 
 def qdump__std____debug__unordered_map(d, value):
     qdump__std__unordered_map(d, value)
@@ -481,22 +636,42 @@ def qdump__std__unordered_set(d, value):
             ptrSize = d.ptrSize()
             for i in d.childRange():
                 d.putSubItem(i, d.createValue(p + ptrSize - offset, valueType))
-                p = d.dereference(p + offset)
+                p = d.extractPointer(p + offset)
 
 def qform__std____1__unordered_map():
     return mapForms()
 
 def qdump__std____1__unordered_map(d, value):
-    n = toInteger(value["__table_"]["__p2_"]["__first_"])
-    d.putItemCount(n)
+    size = int(value["__table_"]["__p2_"]["__first_"])
+    d.putItemCount(size)
     if d.isExpanded():
-        with Children(d, 1):
-            d.putFields(value)
+        node = value["__table_"]["__p1_"]["__first_"]["__next_"]
+        with PairedChildren(d, size, pairType=node["__value_"].type, maxNumChild=1000):
+            for i in d.childRange():
+                with SubItem(d, i):
+                    d.putPair(node["__value_"], i)
+                node = node["__next_"]
+
+
+def qdump__std____1__unordered_set(d, value):
+    size = int(value["__table_"]["__p2_"]["__first_"])
+    d.putItemCount(size)
+    if d.isExpanded():
+        node = value["__table_"]["__p1_"]["__first_"]["__next_"]
+        valueType = d.templateArgument(value.type, 0)
+        with Children(d, size, childType=valueType, maxNumChild=1000):
+            for i in d.childRange():
+                d.putSubItem(i, node["__value_"])
+                node = node["__next_"]
+
 
 def qdump__std____debug__unordered_set(d, value):
     qdump__std__unordered_set(d, value)
 
 
+def qform__std__vector():
+    return arrayForms()
+
 def qedit__std__vector(d, value, data):
     import gdb
     values = data.split(',')
@@ -510,6 +685,10 @@ def qedit__std__vector(d, value, data):
     gdb.execute(cmd)
 
 def qdump__std__vector(d, value):
+    if d.isQnxTarget():
+        qdump__std__vector__QNX(d, value)
+        return
+
     impl = value["_M_impl"]
     type = d.templateArgument(value.type, 0)
     alloc = impl["_M_end_of_storage"]
@@ -543,8 +722,41 @@ def qdump__std__vector(d, value):
                 base = d.pointerValue(start)
                 for i in d.childRange():
                     q = base + int(i / 8)
-                    d.putBoolItem(str(i),
-                        (int(d.dereference(q)) >> (i % 8)) & 1)
+                    d.putBoolItem(str(i), (int(d.extractPointer(q)) >> (i % 8)) & 1)
+        else:
+            d.putPlotData(type, start, size)
+
+def qdump__std__vector__QNX(d, value):
+    type = d.templateArgument(value.type, 0)
+    isBool = str(type) == 'bool'
+    if isBool:
+        impl = value['_Myvec']
+        start = impl['_Myfirst']
+        last = impl['_Mylast']
+        end = impl['_Myend']
+        size = value['_Mysize']
+        storagesize = start.dereference().type.sizeof * 8
+    else:
+        start = value['_Myfirst']
+        last = value['_Mylast']
+        end = value['_Myend']
+        size = int (last - start)
+        alloc = int (end - start)
+
+    d.check(0 <= size and size <= 1000 * 1000 * 1000)
+    d.check(last <= end)
+    d.checkPointer(start)
+    d.checkPointer(last)
+    d.checkPointer(end)
+
+    d.putItemCount(size)
+    d.putNumChild(size)
+    if d.isExpanded():
+        if isBool:
+            with Children(d, size, maxNumChild=10000, childType=type):
+                for i in d.childRange():
+                    q = start + int(i / storagesize)
+                    d.putBoolItem(str(i), (q.dereference() >> (i % storagesize)) & 1)
         else:
             d.putArrayData(type, start, size)
 
@@ -552,7 +764,7 @@ def qdump__std____1__vector(d, value):
     innerType = d.templateArgument(value.type, 0)
     if d.isLldb and d.childAt(value, 0).type == innerType:
         # That's old lldb automatically formatting
-        begin = d.dereferenceValue(value)
+        begin = d.extractPointer(value)
         size = value.GetNumChildren()
     else:
         # Normal case
@@ -563,12 +775,16 @@ def qdump__std____1__vector(d, value):
     d.putItemCount(size)
     d.putNumChild(size)
     if d.isExpanded():
-        d.putArrayData(innerType, begin, size)
+        d.putPlotData(innerType, begin, size)
 
 
+def qform__std____debug__vector():
+    return arrayForms()
+
 def qdump__std____debug__vector(d, value):
     qdump__std__vector(d, value)
 
+
 def qedit__std__string(d, value, data):
     d.call(value, "assign", '"%s"' % data.replace('"', '\\"'))
 
@@ -586,6 +802,15 @@ def qdump__std__basic_string(d, value):
     innerType = d.templateArgument(value.type, 0)
     qdump__std__stringHelper1(d, value, innerType.sizeof)
 
+def qdump__std____1__basic_string(d, value):
+    innerType = str(d.templateArgument(value.type, 0))
+    if innerType == "char":
+        qdump__std____1__string(d, value)
+    elif innerType == "wchar_t":
+        qdump__std____1__wstring(d, value)
+    else:
+        warn("UNKNOWN INNER TYPE %s" % innerType)
+
 def qdump__wstring(d, value):
     qdump__std__wstring(d, value)
 
