@@ -251,7 +251,7 @@ public:
 protected:
     virtual void visitSymbol0(SymbolVisitor *visitor);
     virtual void accept0(TypeVisitor *visitor);
-    virtual bool matchType0(const Type *otherType, TypeMatcher *matcher) const;
+    virtual bool match0(const Type *otherType, Matcher *matcher) const;
 };
 
 class CPLUSPLUS_EXPORT Enum: public Scope, public Type
@@ -285,7 +285,7 @@ public:
 protected:
     virtual void visitSymbol0(SymbolVisitor *visitor);
     virtual void accept0(TypeVisitor *visitor);
-    virtual bool matchType0(const Type *otherType, TypeMatcher *matcher) const;
+    virtual bool match0(const Type *otherType, Matcher *matcher) const;
 
 private:
     bool _isScoped;
@@ -347,6 +347,8 @@ public:
     bool isPureVirtual() const;
     void setPureVirtual(bool isPureVirtual);
 
+    bool isSignatureEqualTo(const Function *other) const;
+
     // Symbol's interface
     virtual FullySpecifiedType type() const;
 
@@ -373,7 +375,7 @@ public:
 protected:
     virtual void visitSymbol0(SymbolVisitor *visitor);
     virtual void accept0(TypeVisitor *visitor);
-    virtual bool matchType0(const Type *otherType, TypeMatcher *matcher) const;
+    virtual bool match0(const Type *otherType, Matcher *matcher) const;
 
 private:
     FullySpecifiedType _returnType;
@@ -426,7 +428,7 @@ public:
 protected:
     virtual void visitSymbol0(SymbolVisitor *visitor);
     virtual void accept0(TypeVisitor *visitor);
-    virtual bool matchType0(const Type *otherType, TypeMatcher *matcher) const;
+    virtual bool match0(const Type *otherType, Matcher *matcher) const;
 };
 
 
@@ -464,7 +466,7 @@ public:
 protected:
     virtual void visitSymbol0(SymbolVisitor *visitor);
     virtual void accept0(TypeVisitor *visitor);
-    virtual bool matchType0(const Type *otherType, TypeMatcher *matcher) const;
+    virtual bool match0(const Type *otherType, Matcher *matcher) const;
 
 private:
     bool _isInline;
@@ -542,7 +544,7 @@ public:
 protected:
     virtual void visitSymbol0(SymbolVisitor *visitor);
     virtual void accept0(TypeVisitor *visitor);
-    virtual bool matchType0(const Type *otherType, TypeMatcher *matcher) const;
+    virtual bool match0(const Type *otherType, Matcher *matcher) const;
 
 private:
     Key _key;
@@ -684,7 +686,7 @@ public:
 protected:
     virtual void visitSymbol0(SymbolVisitor *visitor);
     virtual void accept0(TypeVisitor *visitor);
-    virtual bool matchType0(const Type *otherType, TypeMatcher *matcher) const;
+    virtual bool match0(const Type *otherType, Matcher *matcher) const;
 };
 
 class CPLUSPLUS_EXPORT ObjCProtocol: public Scope, public Type
@@ -719,7 +721,7 @@ public:
 protected:
     virtual void visitSymbol0(SymbolVisitor *visitor);
     virtual void accept0(TypeVisitor *visitor);
-    virtual bool matchType0(const Type *otherType, TypeMatcher *matcher) const;
+    virtual bool match0(const Type *otherType, Matcher *matcher) const;
 
 private:
     std::vector<ObjCBaseProtocol *> _protocols;
@@ -751,7 +753,7 @@ public:
 protected:
     virtual void visitSymbol0(SymbolVisitor *visitor);
     virtual void accept0(TypeVisitor *visitor);
-    virtual bool matchType0(const Type *otherType, TypeMatcher *matcher) const;
+    virtual bool match0(const Type *otherType, Matcher *matcher) const;
 };
 
 class CPLUSPLUS_EXPORT ObjCClass: public Scope, public Type
@@ -796,7 +798,7 @@ public:
 protected:
     virtual void visitSymbol0(SymbolVisitor *visitor);
     virtual void accept0(TypeVisitor *visitor);
-    virtual bool matchType0(const Type *otherType, TypeMatcher *matcher) const;
+    virtual bool match0(const Type *otherType, Matcher *matcher) const;
 
 private:
     const Name *_categoryName;
@@ -848,7 +850,7 @@ public:
 protected:
     virtual void visitSymbol0(SymbolVisitor *visitor);
     virtual void accept0(TypeVisitor *visitor);
-    virtual bool matchType0(const Type *otherType, TypeMatcher *matcher) const;
+    virtual bool match0(const Type *otherType, Matcher *matcher) const;
 
 private:
     FullySpecifiedType _returnType;
