@@ -92,6 +92,7 @@ CMakeProject::CMakeProject(CMakeManager *manager, const QString &fileName)
       m_rootNode(new CMakeProjectNode(fileName)),
       m_watcher(new QFileSystemWatcher(this))
 {
+    setId(Constants::CMAKEPROJECT_ID);
     setProjectContext(Core::Context(CMakeProjectManager::Constants::PROJECTCONTEXT));
     setProjectLanguages(Core::Context(ProjectExplorer::Constants::LANG_CXX));
 
@@ -142,7 +143,7 @@ void CMakeProject::changeActiveBuildConfiguration(ProjectExplorer::BuildConfigur
 
     if (mode != CMakeOpenProjectWizard::Nothing) {
         CMakeBuildInfo info(cmakebc);
-        CMakeOpenProjectWizard copw(m_manager, mode, &info);
+        CMakeOpenProjectWizard copw(Core::ICore::mainWindow(), m_manager, mode, &info);
         if (copw.exec() == QDialog::Accepted)
             cmakebc->setUseNinja(copw.useNinja()); // NeedToCreate can change the Ninja setting
     }
@@ -341,20 +342,35 @@ bool CMakeProject::parseCMakeLists()
         part->displayName = displayName();
         part->projectFile = projectFilePath();
 
-        part->evaluateToolchain(tc,
-                                cxxflags,
-                                cxxflags,
-                                SysRootKitInformation::sysRoot(k));
-
         // This explicitly adds -I. to the include paths
         part->includePaths += projectDirectory();
-        part->includePaths += cbpparser.includeFiles();
-        part->defines += cbpparser.defines();
+
+        foreach (const QString &includeFile, cbpparser.includeFiles()) {
+            // CodeBlocks is utterly ignorant of frameworks on Mac, and won't report framework
+            // paths. The work-around is to check if the include path ends in ".framework", and
+            // if so, add the parent directory as framework path.
+            if (includeFile.endsWith(QLatin1String(".framework"))) {
+                const int slashIdx = includeFile.lastIndexOf(QLatin1Char('/'));
+                if (slashIdx != -1) {
+                    part->frameworkPaths += includeFile.left(slashIdx);
+                    continue;
+                }
+            }
+
+            part->includePaths += includeFile;
+        }
+
+        part->projectDefines += cbpparser.defines();
 
         CppTools::ProjectFileAdder adder(part->files);
         foreach (const QString &file, m_files)
             adder.maybeAdd(file);
 
+        part->evaluateToolchain(tc,
+                                cxxflags,
+                                cxxflags,
+                                SysRootKitInformation::sysRoot(k));
+
         pinfo.appendProjectPart(part);
         m_codeModelFuture.cancel();
         m_codeModelFuture = modelmanager->updateProjectInfo(pinfo);
@@ -419,38 +435,12 @@ void CMakeProject::buildTree(CMakeProjectNode *rootNode, QList<ProjectExplorer::
     qSort(oldList.begin(), oldList.end(), sortNodesByPath);
     qSort(newList.begin(), newList.end(), sortNodesByPath);
 
-    // generate added and deleted list
-    QList<ProjectExplorer::FileNode *>::const_iterator oldIt  = oldList.constBegin();
-    QList<ProjectExplorer::FileNode *>::const_iterator oldEnd = oldList.constEnd();
-    QList<ProjectExplorer::FileNode *>::const_iterator newIt  = newList.constBegin();
-    QList<ProjectExplorer::FileNode *>::const_iterator newEnd = newList.constEnd();
-
     QList<ProjectExplorer::FileNode *> added;
     QList<ProjectExplorer::FileNode *> deleted;
 
-    while (oldIt != oldEnd && newIt != newEnd) {
-        if ( (*oldIt)->path() == (*newIt)->path()) {
-            delete *newIt;
-            ++oldIt;
-            ++newIt;
-        } else if ((*oldIt)->path() < (*newIt)->path()) {
-            deleted.append(*oldIt);
-            ++oldIt;
-        } else {
-            added.append(*newIt);
-            ++newIt;
-        }
-    }
+    ProjectExplorer::compareSortedLists(oldList, newList, deleted, added, sortNodesByPath);
 
-    while (oldIt != oldEnd) {
-        deleted.append(*oldIt);
-        ++oldIt;
-    }
-
-    while (newIt != newEnd) {
-        added.append(*newIt);
-        ++newIt;
-    }
+    qDeleteAll(ProjectExplorer::subtractSortedList(newList, added, sortNodesByPath));
 
     // add added nodes
     foreach (ProjectExplorer::FileNode *fn, added) {
@@ -458,18 +448,18 @@ void CMakeProject::buildTree(CMakeProjectNode *rootNode, QList<ProjectExplorer::
         // Get relative path to rootNode
         QString parentDir = QFileInfo(fn->path()).absolutePath();
         ProjectExplorer::FolderNode *folder = findOrCreateFolder(rootNode, parentDir);
-        rootNode->addFileNodes(QList<ProjectExplorer::FileNode *>()<< fn, folder);
+        folder->addFileNodes(QList<ProjectExplorer::FileNode *>()<< fn);
     }
 
     // remove old file nodes and check whether folder nodes can be removed
     foreach (ProjectExplorer::FileNode *fn, deleted) {
         ProjectExplorer::FolderNode *parent = fn->parentFolderNode();
 //        qDebug()<<"removed"<<fn->path();
-        rootNode->removeFileNodes(QList<ProjectExplorer::FileNode *>() << fn, parent);
+        parent->removeFileNodes(QList<ProjectExplorer::FileNode *>() << fn);
         // Check for empty parent
         while (parent->subFolderNodes().isEmpty() && parent->fileNodes().isEmpty()) {
             ProjectExplorer::FolderNode *grandparent = parent->parentFolderNode();
-            rootNode->removeFolderNodes(QList<ProjectExplorer::FolderNode *>() << parent, grandparent);
+            grandparent->removeFolderNodes(QList<ProjectExplorer::FolderNode *>() << parent);
             parent = grandparent;
             if (parent == rootNode)
                 break;
@@ -500,7 +490,7 @@ ProjectExplorer::FolderNode *CMakeProject::findOrCreateFolder(CMakeProjectNode *
             // No FolderNode yet, so create it
             ProjectExplorer::FolderNode *tmp = new ProjectExplorer::FolderNode(path);
             tmp->setDisplayName(part);
-            rootNode->addFolderNodes(QList<ProjectExplorer::FolderNode *>() << tmp, parent);
+            parent->addFolderNodes(QList<ProjectExplorer::FolderNode *>() << tmp);
             parent = tmp;
         }
     }
@@ -512,11 +502,6 @@ QString CMakeProject::displayName() const
     return m_projectName;
 }
 
-Core::Id CMakeProject::id() const
-{
-    return Core::Id(Constants::CMAKEPROJECT_ID);
-}
-
 Core::IDocument *CMakeProject::document() const
 {
     return m_file;
@@ -546,7 +531,7 @@ bool CMakeProject::fromMap(const QVariantMap &map)
 
     bool hasUserFile = activeTarget();
     if (!hasUserFile) {
-        CMakeOpenProjectWizard copw(m_manager, projectDirectory(), Utils::Environment::systemEnvironment());
+        CMakeOpenProjectWizard copw(Core::ICore::mainWindow(), m_manager, projectDirectory(), Utils::Environment::systemEnvironment());
         if (copw.exec() != QDialog::Accepted)
             return false;
         Kit *k = copw.kit();
@@ -589,7 +574,7 @@ bool CMakeProject::fromMap(const QVariantMap &map)
 
         if (mode != CMakeOpenProjectWizard::Nothing) {
             CMakeBuildInfo info(activeBC);
-            CMakeOpenProjectWizard copw(m_manager, mode, &info);
+            CMakeOpenProjectWizard copw(Core::ICore::mainWindow(), m_manager, mode, &info);
             if (copw.exec() != QDialog::Accepted)
                 return false;
             else
@@ -871,7 +856,8 @@ void CMakeBuildSettingsWidget::openChangeBuildDirectoryDialog()
 {
     CMakeProject *project = static_cast<CMakeProject *>(m_buildConfiguration->target()->project());
     CMakeBuildInfo info(m_buildConfiguration);
-    CMakeOpenProjectWizard copw(project->projectManager(), CMakeOpenProjectWizard::ChangeDirectory,
+    CMakeOpenProjectWizard copw(Core::ICore::mainWindow(),
+                                project->projectManager(), CMakeOpenProjectWizard::ChangeDirectory,
                                 &info);
     if (copw.exec() == QDialog::Accepted) {
         project->changeBuildDirectory(m_buildConfiguration, copw.buildDirectory());
@@ -886,7 +872,8 @@ void CMakeBuildSettingsWidget::runCMake()
         return;
     CMakeProject *project = static_cast<CMakeProject *>(m_buildConfiguration->target()->project());
     CMakeBuildInfo info(m_buildConfiguration);
-    CMakeOpenProjectWizard copw(project->projectManager(),
+    CMakeOpenProjectWizard copw(Core::ICore::mainWindow(),
+                                project->projectManager(),
                                 CMakeOpenProjectWizard::WantToUpdate, &info);
     if (copw.exec() == QDialog::Accepted)
         project->parseCMakeLists();
