@@ -1045,7 +1045,7 @@ void DebuggerEnginePrivate::doFinishDebugger()
 
 void DebuggerEnginePrivate::setRemoteSetupState(RemoteSetupState state)
 {
-    bool allowedTransition = true;
+    bool allowedTransition = false;
     if (m_remoteSetupState == RemoteSetupNone) {
         if (state == RemoteSetupRequested)
             allowedTransition = true;
@@ -1402,6 +1402,10 @@ void DebuggerEngine::reloadFullStack()
 {
 }
 
+void DebuggerEngine::loadAdditionalQmlStack()
+{
+}
+
 void DebuggerEngine::reloadDebuggingHelpers()
 {
 }
@@ -1800,7 +1804,7 @@ void DebuggerEngine::checkForReleaseBuild(const DebuggerStartParameters &sp)
 
         foreach (const QByteArray &name, interesting) {
             const QString found = seen.contains(name) ? tr("Found.") : tr("Not found.");
-            detailedWarning.append(tr("\nSection %1: %2").arg(_(name)).arg(found));
+            detailedWarning.append(QLatin1Char('\n') + tr("Section %1: %2").arg(_(name)).arg(found));
         }
         break;
     }
