@@ -51,7 +51,7 @@
 #include <coreplugin/actionmanager/actioncontainer.h>
 #include <coreplugin/actionmanager/command.h>
 #include <coreplugin/editormanager/editormanager.h>
-#include <locator/commandlocator.h>
+#include <coreplugin/locator/commandlocator.h>
 
 #include <vcsbase/vcsbaseoutputwindow.h>
 
@@ -256,7 +256,7 @@ void FetchContext::cherryPick()
 
 void FetchContext::checkout()
 {
-    Git::Internal::GitPlugin::instance()->gitClient()->synchronousCheckout(
+    Git::Internal::GitPlugin::instance()->gitClient()->stashAndCheckout(
                 m_repository, QLatin1String("FETCH_HEAD"));
 }
 
@@ -302,7 +302,7 @@ void GerritPlugin::updateActions(bool hasTopLevel)
     m_pushToGerritPair.first->setEnabled(hasTopLevel);
 }
 
-void GerritPlugin::addToLocator(Locator::CommandLocator *locator)
+void GerritPlugin::addToLocator(Core::CommandLocator *locator)
 {
     locator->appendCommand(m_gerritCommand);
     locator->appendCommand(m_pushToGerritPair.second);
@@ -313,9 +313,6 @@ void GerritPlugin::push(const QString &topLevel)
     // QScopedPointer is required to delete the dialog when leaving the function
     GerritPushDialog dialog(topLevel, m_reviewers, ICore::mainWindow());
 
-    if (!dialog.localChangesFound())
-        return;
-
     if (!dialog.valid()) {
         QMessageBox::warning(ICore::mainWindow(), tr("Initialization Failed"),
                               tr("Failed to initialize dialog. Aborting."));
@@ -359,7 +356,7 @@ void GerritPlugin::openView()
 {
     if (m_dialog.isNull()) {
         while (!m_parameters->isValid()) {
-            QMessageBox::warning(0, tr("Error"),
+            QMessageBox::warning(ICore::dialogParent(), tr("Error"),
                     tr("Invalid Gerrit configuration. Host, user and ssh binary are mandatory."));
             const Id group = VcsBase::Constants::VCS_SETTINGS_CATEGORY;
             if (!ICore::showOptionsDialog(group, "Gerrit"))
