@@ -65,7 +65,7 @@
 #include <coreplugin/rightpane.h>
 #include <coreplugin/sidebar.h>
 #include <extensionsystem/pluginmanager.h>
-#include <find/findplugin.h>
+#include <coreplugin/find/findplugin.h>
 #include <texteditor/texteditorconstants.h>
 #include <utils/hostosinfo.h>
 #include <utils/styledbar.h>
@@ -328,7 +328,7 @@ bool HelpPlugin::initialize(const QStringList &arguments, QString *error)
         layout->setSpacing(0);
         FindToolBarPlaceHolder *fth = new FindToolBarPlaceHolder(m_centralWidget);
         fth->setObjectName(QLatin1String("HelpFindToolBarPlaceHolder"));
-        layout->addWidget(fth);
+        mainWidgetLayout->addWidget(fth);
     }
 
     HelpIndexFilter *helpIndexFilter = new HelpIndexFilter();
@@ -874,9 +874,12 @@ void HelpPlugin::activateContext()
         links = HelpManager::linksForIdentifier(m_idFromContext);
         if (links.isEmpty()) {
             // Maybe this is already an URL...
-            QUrl url(m_idFromContext);
-            if (url.isValid())
-                links.insert(m_idFromContext, m_idFromContext);
+            // Require protocol specifier, otherwise most strings would be 'local file names'
+            if (m_idFromContext.contains(QLatin1Char(':'))) {
+                QUrl url(m_idFromContext);
+                if (url.isValid())
+                    links.insert(m_idFromContext, m_idFromContext);
+            }
         }
     }
 
@@ -1131,7 +1134,7 @@ void HelpPlugin::handleHelpRequest(const QUrl &url)
             || address.startsWith(QLatin1String("qthelp://com.nokia."))
             || address.startsWith(QLatin1String("qthelp://com.trolltech."))) {
                 // local help not installed, resort to external web help
-                QString urlPrefix = QLatin1String("http://doc.qt.digia.com/");
+                QString urlPrefix = QLatin1String("http://qt-project.org/doc/");
                 if (url.authority() == QLatin1String("org.qt-project.qtcreator"))
                     urlPrefix.append(QString::fromLatin1("qtcreator"));
                 else
@@ -1213,8 +1216,8 @@ void HelpPlugin::slotReportBug()
 
 void HelpPlugin::openFindToolBar()
 {
-    if (Find::FindPlugin::instance())
-        Find::FindPlugin::instance()->openFindToolBar(Find::FindPlugin::FindForwardDirection);
+    if (FindPlugin::instance())
+        FindPlugin::instance()->openFindToolBar(FindPlugin::FindForwardDirection);
 }
 
 void  HelpPlugin::onSideBarVisibilityChanged()
