@@ -34,6 +34,7 @@
 #include <extensionsystem/pluginmanager.h>
 #include <projectexplorer/project.h>
 #include <projectexplorer/projectexplorerconstants.h>
+#include <utils/qtcassert.h>
 #include <utils/fileutils.h>
 #include <utils/persistentsettings.h>
 #include <utils/qtcassert.h>
@@ -81,11 +82,11 @@ DeviceManager *DeviceManagerPrivate::clonedInstance = 0;
 
 using namespace Internal;
 
+DeviceManager *DeviceManager::m_instance = 0;
 
 DeviceManager *DeviceManager::instance()
 {
-    static DeviceManager instance;
-    return &instance;
+    return m_instance;
 }
 
 int DeviceManager::deviceCount() const
@@ -141,7 +142,7 @@ void DeviceManager::load()
 
     // Only create writer now: We do not want to save before the settings were read!
     d->writer = new Utils::PersistentSettingsWriter(
-                settingsFilePath(QLatin1String("/qtcreator/devices.xml")),
+                settingsFilePath(QLatin1String("/devices.xml")),
                 QLatin1String("QtCreatorDevices"));
 
     Utils::PersistentSettingsReader reader;
@@ -151,7 +152,7 @@ void DeviceManager::load()
         sdkDevices = fromMap(reader.restoreValues().value(QLatin1String(DeviceManagerKey)).toMap());
     // read devices file from user settings path
     QList<IDevice::Ptr> userDevices;
-    if (reader.load(settingsFilePath(QLatin1String("/qtcreator/devices.xml"))))
+    if (reader.load(settingsFilePath(QLatin1String("/devices.xml"))))
         userDevices = fromMap(reader.restoreValues().value(QLatin1String(DeviceManagerKey)).toMap());
     // Insert devices into the model. Prefer the higher device version when there are multiple
     // devices with the same id.
@@ -215,7 +216,7 @@ QVariantMap DeviceManager::toMap() const
 
 Utils::FileName DeviceManager::settingsFilePath(const QString &extension)
 {
-    return Utils::FileName::fromString(QFileInfo(Core::ICore::settings()->fileName()).absolutePath() + extension);
+    return Utils::FileName::fromString(Core::ICore::userResourcePath() + extension);
 }
 
 Utils::FileName DeviceManager::systemSettingsFilePath(const QString &deviceFileRelativePath)
@@ -337,14 +338,19 @@ const IDeviceFactory *DeviceManager::restoreFactory(const QVariantMap &map)
 
 DeviceManager::DeviceManager(bool isInstance) : d(new DeviceManagerPrivate)
 {
-    if (isInstance)
+    if (isInstance) {
+        QTC_ASSERT(!m_instance, return);
+        m_instance = this;
         connect(Core::ICore::instance(), SIGNAL(saveSettingsRequested()), SLOT(save()));
+    }
 }
 
 DeviceManager::~DeviceManager()
 {
     if (d->clonedInstance != this)
         delete d->writer;
+    if (m_instance == this)
+        m_instance = 0;
     delete d;
 }
 
