@@ -51,6 +51,8 @@
 #include <qtsupport/qtkitinformation.h>
 #include <qtsupport/uicodemodelsupport.h>
 
+#include <resourceeditor/resourcenode.h>
+
 #include <cpptools/cppmodelmanagerinterface.h>
 #include <cpptools/cpptoolsconstants.h>
 
@@ -100,10 +102,18 @@ static const FileTypeDataStorage fileTypeDataStorage[] = {
       ":/qmakeprojectmanager/images/unknown.png" }
 };
 
-bool sortNodesByPath(ProjectExplorer::Node *a, ProjectExplorer::Node *b)
+class SortByPath
 {
-    return a->path() < b->path();
-}
+public:
+    bool operator()(ProjectExplorer::Node *a, ProjectExplorer::Node *b)
+    { return operator()(a->path(), b->path()); }
+    bool operator()(ProjectExplorer::Node *a, const QString &b)
+    { return operator()(a->path(), b); }
+    bool operator()(const QString &a, ProjectExplorer::Node *b)
+    { return operator()(a, b->path()); }
+    bool operator()(const QString &a, const QString &b)
+    { return a < b; }
+};
 
 class QmakeNodeStaticData {
 public:
@@ -384,14 +394,17 @@ struct InternalNode
     }
 
     // Makes the projectNode's subtree below the given folder match this internal node's subtree
-    void updateSubFolders(QmakeProjectManager::QmakePriFileNode *projectNode, ProjectExplorer::FolderNode *folder)
+    void updateSubFolders(ProjectExplorer::FolderNode *folder)
     {
-        updateFiles(projectNode, folder, type);
+        if (type == ProjectExplorer::ResourceType)
+            updateResourceFiles(folder);
+        else
+            updateFiles(folder, type);
 
         // updateFolders
         QMultiMap<QString, FolderNode *> existingFolderNodes;
         foreach (FolderNode *node, folder->subFolderNodes())
-            if (node->nodeType() != ProjectNodeType)
+            if (node->nodeType() != ProjectNodeType && !qobject_cast<ResourceEditor::ResourceTopLevelNode *>(node))
                 existingFolderNodes.insert(node->path(), node);
 
         QList<FolderNode *> foldersToRemove;
@@ -466,16 +479,16 @@ struct InternalNode
                 foldersToRemove << jit.value();
 
         if (!foldersToRemove.isEmpty())
-            projectNode->removeFolderNodes(foldersToRemove, folder);
+            folder->removeFolderNodes(foldersToRemove);
         if (!foldersToAdd.isEmpty())
-            projectNode->addFolderNodes(foldersToAdd, folder);
+            folder->addFolderNodes(foldersToAdd);
 
         foreach (const NodePair &np, nodesToUpdate)
-            np.first->updateSubFolders(projectNode, np.second);
+            np.first->updateSubFolders(np.second);
     }
 
     // Makes the folder's files match this internal node's file list
-    void updateFiles(QmakeProjectManager::QmakePriFileNode *projectNode, FolderNode *folder, FileType type)
+    void updateFiles(FolderNode *folder, FileType type)
     {
         QList<FileNode*> existingFileNodes;
         foreach (FileNode *fileNode, folder->fileNodes()) {
@@ -484,39 +497,51 @@ struct InternalNode
         }
 
         QList<FileNode*> filesToRemove;
-        QList<FileNode*> filesToAdd;
+        QStringList filesToAdd;
 
-        qSort(files);
-        qSort(existingFileNodes.begin(), existingFileNodes.end(), sortNodesByPath);
+        SortByPath sortByPath;
+        qSort(files.begin(), files.end(), sortByPath);
+        qSort(existingFileNodes.begin(), existingFileNodes.end(), sortByPath);
 
-        QList<FileNode*>::const_iterator existingNodeIter = existingFileNodes.constBegin();
-        QList<QString>::const_iterator newPathIter = files.constBegin();
-        while (existingNodeIter != existingFileNodes.constEnd()
-               && newPathIter != files.constEnd()) {
-            if ((*existingNodeIter)->path() < *newPathIter) {
-                filesToRemove << *existingNodeIter;
-                ++existingNodeIter;
-            } else if ((*existingNodeIter)->path() > *newPathIter) {
-                filesToAdd << new ProjectExplorer::FileNode(*newPathIter, type, false);
-                ++newPathIter;
-            } else { // *existingNodeIter->path() == *newPathIter
-                ++existingNodeIter;
-                ++newPathIter;
-            }
-        }
-        while (existingNodeIter != existingFileNodes.constEnd()) {
-            filesToRemove << *existingNodeIter;
-            ++existingNodeIter;
-        }
-        while (newPathIter != files.constEnd()) {
-            filesToAdd << new ProjectExplorer::FileNode(*newPathIter, type, false);
-            ++newPathIter;
+        ProjectExplorer::compareSortedLists(existingFileNodes, files, filesToRemove, filesToAdd, sortByPath);
+
+        QList<FileNode *> nodesToAdd;
+        foreach (const QString &file, filesToAdd)
+            nodesToAdd << new ProjectExplorer::FileNode(file, type, false);
+
+        folder->removeFileNodes(filesToRemove);
+        folder->addFileNodes(nodesToAdd);
+    }
+
+    // Makes the folder's files match this internal node's file list
+    void updateResourceFiles(FolderNode *folder)
+    {
+        QList<ProjectExplorer::FolderNode *> existingResourceNodes; // for resource special handling
+        foreach (FolderNode *folderNode, folder->subFolderNodes()) {
+            if (ResourceEditor::ResourceTopLevelNode *rn = qobject_cast<ResourceEditor::ResourceTopLevelNode *>(folderNode))
+                existingResourceNodes << rn;
         }
 
-        if (!filesToRemove.isEmpty())
-            projectNode->removeFileNodes(filesToRemove, folder);
-        if (!filesToAdd.isEmpty())
-            projectNode->addFileNodes(filesToAdd, folder);
+        QList<ProjectExplorer::FolderNode *> resourcesToRemove;
+        QStringList resourcesToAdd;
+
+        SortByPath sortByPath;
+        qSort(files.begin(), files.end(), sortByPath);
+        qSort(existingResourceNodes.begin(), existingResourceNodes.end(), sortByPath);
+
+        ProjectExplorer::compareSortedLists(existingResourceNodes, files, resourcesToRemove, resourcesToAdd, sortByPath);
+
+        QList<FolderNode *> nodesToAdd;
+        nodesToAdd.reserve(resourcesToAdd.size());
+
+        foreach (const QString &file, resourcesToAdd)
+            nodesToAdd.append(new ResourceEditor::ResourceTopLevelNode(file, folder));
+
+        folder->removeFolderNodes(resourcesToRemove);
+        folder->addFolderNodes(nodesToAdd);
+
+        foreach (FolderNode *fn, nodesToAdd)
+            qobject_cast<ResourceEditor::ResourceTopLevelNode *>(fn)->update();
     }
 };
 }
@@ -569,7 +594,7 @@ void QmakePriFileNode::update(ProFile *includeFileExact, QtSupport::ProFileReade
 {
     // add project file node
     if (m_fileNodes.isEmpty())
-        addFileNodes(QList<FileNode*>() << new ProjectExplorer::FileNode(m_projectFilePath, ProjectExplorer::ProjectFileType, false), this);
+        addFileNodes(QList<FileNode*>() << new ProjectExplorer::FileNode(m_projectFilePath, ProjectExplorer::ProjectFileType, false));
 
     const QString &projectDir = m_qmakeProFileNode->m_projectDir;
 
@@ -640,7 +665,7 @@ void QmakePriFileNode::update(ProFile *includeFileExact, QtSupport::ProFileReade
     QFileInfo tmpFi;
     for (int i = 0; i < fileTypes.size(); ++i) {
         FileType type = fileTypes.at(i).type;
-        QStringList qmakeVariables = varNames(type);
+        QStringList qmakeVariables = varNames(type, readerExact);
 
         QSet<Utils::FileName> newFilePaths;
         foreach (const QString &qmakeVariable, qmakeVariables) {
@@ -695,7 +720,7 @@ void QmakePriFileNode::update(ProFile *includeFileExact, QtSupport::ProFileReade
         }
     }
 
-    contents.updateSubFolders(this, this);
+    contents.updateSubFolders(this);
 }
 
 void QmakePriFileNode::watchFolders(const QSet<QString> &folders)
@@ -772,7 +797,7 @@ bool QmakePriFileNode::folderChanged(const QString &changedFolder, const QSet<Ut
         }
     }
 
-    contents.updateSubFolders(this, this);
+    contents.updateSubFolders(this);
     return true;
 }
 
@@ -825,9 +850,9 @@ void QmakePriFileNode::setIncludedInExactParse(bool b)
     m_includedInExactParse = b;
 }
 
-QList<ProjectNode::ProjectAction> QmakePriFileNode::supportedActions(Node *node) const
+QList<ProjectExplorer::ProjectAction> QmakePriFileNode::supportedActions(Node *node) const
 {
-    QList<ProjectAction> actions;
+    QList<ProjectExplorer::ProjectAction> actions;
 
     const FolderNode *folderNode = this;
     const QmakeProFileNode *proFileNode;
@@ -843,11 +868,11 @@ QList<ProjectNode::ProjectAction> QmakePriFileNode::supportedActions(Node *node)
         // projects (e.g. cpp). It'd be nice if the "add" action could
         // work on a subset of the file types according to project type.
 
-        actions << AddNewFile;
+        actions << ProjectExplorer::AddNewFile;
         if (m_recursiveEnumerateFiles.contains(Utils::FileName::fromString(node->path())))
-            actions << EraseFile;
+            actions << ProjectExplorer::EraseFile;
         else
-            actions << RemoveFile;
+            actions << ProjectExplorer::RemoveFile;
 
         bool addExistingFiles = true;
         if (node->nodeType() == ProjectExplorer::VirtualFolderNodeType) {
@@ -865,26 +890,27 @@ QList<ProjectNode::ProjectAction> QmakePriFileNode::supportedActions(Node *node)
         addExistingFiles = addExistingFiles && !deploysFolder(node->path());
 
         if (addExistingFiles)
-            actions << AddExistingFile;
+            actions << ProjectExplorer::AddExistingFile << ProjectExplorer::AddExistingDirectory;
 
         break;
     }
     case SubDirsTemplate:
-        actions << AddSubProject << RemoveSubProject;
+        actions << ProjectExplorer::AddSubProject << ProjectExplorer::RemoveSubProject;
         break;
     default:
         break;
     }
 
     ProjectExplorer::FileNode *fileNode = qobject_cast<FileNode *>(node);
-    if (fileNode && fileNode->fileType() != ProjectExplorer::ProjectFileType)
-        actions << Rename;
+    if ((fileNode && fileNode->fileType() != ProjectExplorer::ProjectFileType)
+            || qobject_cast<ResourceEditor::ResourceTopLevelNode *>(node))
+        actions << ProjectExplorer::Rename;
 
 
     ProjectExplorer::Target *target = m_project->activeTarget();
     QmakeRunConfigurationFactory *factory = QmakeRunConfigurationFactory::find(target);
     if (factory && !factory->runConfigurationsForNode(target, node).isEmpty())
-        actions << HasSubProjectRunConfigurations;
+        actions << ProjectExplorer::HasSubProjectRunConfigurations;
 
     return actions;
 }
@@ -1018,18 +1044,7 @@ bool QmakePriFileNode::removeFiles(const QStringList &filePaths,
 
 bool QmakePriFileNode::deleteFiles(const QStringList &filePaths)
 {
-    QStringList failedFiles;
-    typedef QMap<QString, QStringList> TypeFileMap;
-    // Split into lists by file type and bulk-add them.
-    TypeFileMap typeFileMap;
-    foreach (const QString file, filePaths) {
-        const Core::MimeType mt = Core::MimeDatabase::findByFile(file);
-        typeFileMap[mt.type()] << file;
-    }
-    foreach (const QString &type, typeFileMap.keys()) {
-        const QStringList typeFiles = typeFileMap.value(type);
-        changeFiles(type, typeFiles, &failedFiles, RemoveFromProFile);
-    }
+    removeFiles(filePaths);
     return true;
 }
 
@@ -1051,6 +1066,12 @@ bool QmakePriFileNode::renameFile(const QString &filePath, const QString &newFil
     return true;
 }
 
+ProjectExplorer::FolderNode::AddNewInformation QmakePriFileNode::addNewInformation(const QStringList &files, Node *context) const
+{
+    Q_UNUSED(files)
+    return ProjectExplorer::FolderNode::AddNewInformation(QFileInfo(path()).fileName(), context == this ? 120 : 90);
+}
+
 bool QmakePriFileNode::priFileWritable(const QString &path)
 {
     Core::Internal::ReadOnlyFilesDialog roDialog(path, Core::ICore::mainWindow());
@@ -1065,11 +1086,11 @@ bool QmakePriFileNode::saveModifiedEditors()
     if (!document || !document->isModified())
         return true;
 
-    bool cancelled;
-    Core::DocumentManager::saveModifiedDocuments(QList<Core::IDocument *>() << document, &cancelled,
-                                                 tr("There are unsaved changes for project file %1.").arg(m_projectFilePath));
-    if (cancelled)
+    if (!Core::DocumentManager::saveDocument(document,
+                                             tr("There are unsaved changes for project file %1.")
+                                             .arg(m_projectFilePath))) {
         return false;
+    }
     // force instant reload of ourselves
     QtSupport::ProFileCacheManager::instance()->discardFile(m_projectFilePath);
     m_project->qmakeProjectManager()->notifyChanged(m_projectFilePath);
@@ -1193,7 +1214,7 @@ void QmakePriFileNode::changeFiles(const QString &mimeType,
     includeFile->deref();
 }
 
-bool QmakePriFileNode::setProVariable(const QString &var, const QString &value)
+bool QmakePriFileNode::setProVariable(const QString &var, const QStringList &values, const QString &scope, int flags)
 {
     if (!ensureWriteableProFile(m_projectFilePath))
         return false;
@@ -1202,8 +1223,9 @@ bool QmakePriFileNode::setProVariable(const QString &var, const QString &value)
     ProFile *includeFile = pair.first;
     QStringList lines = pair.second;
 
-    ProWriter::putVarValues(includeFile, &lines, QStringList(value), var,
-                            ProWriter::ReplaceValues | ProWriter::OneLine | ProWriter::AssignOperator);
+    ProWriter::putVarValues(includeFile, &lines, values, var,
+                            ProWriter::PutFlags(flags),
+                            scope);
 
     if (!includeFile)
         return false;
@@ -1238,21 +1260,28 @@ void QmakePriFileNode::save(const QStringList &lines)
                              errorStrings.join(QLatin1String("\n")));
 }
 
-QStringList QmakePriFileNode::varNames(ProjectExplorer::FileType type)
+QStringList QmakePriFileNode::varNames(ProjectExplorer::FileType type, QtSupport::ProFileReader *readerExact)
 {
     QStringList vars;
     switch (type) {
     case ProjectExplorer::HeaderType:
         vars << QLatin1String("HEADERS");
-        vars << QLatin1String("OBJECTIVE_HEADERS");
         vars << QLatin1String("PRECOMPILED_HEADER");
         break;
-    case ProjectExplorer::SourceType:
+    case ProjectExplorer::SourceType: {
         vars << QLatin1String("SOURCES");
-        vars << QLatin1String("OBJECTIVE_SOURCES");
-        vars << QLatin1String("LEXSOURCES");
-        vars << QLatin1String("YACCSOURCES");
+        QStringList listOfExtraCompilers = readerExact->values(QLatin1String("QMAKE_EXTRA_COMPILERS"));
+        foreach (const QString &var, listOfExtraCompilers) {
+            QStringList inputs = readerExact->values(var + QLatin1String(".input"));
+            foreach (const QString &input, inputs)
+                // FORMS and RESOURCES are handled below
+                if (input != QLatin1String("FORMS")
+                        && input != QLatin1String("RESOURCES")
+                        && input != QLatin1String("HEADERS"))
+                    vars << input;
+        }
         break;
+    }
     case ProjectExplorer::ResourceType:
         vars << QLatin1String("RESOURCES");
         break;
@@ -1539,12 +1568,18 @@ bool QmakeProFileNode::isParent(QmakeProFileNode *node)
     return false;
 }
 
-bool QmakeProFileNode::hasBuildTargets() const
+bool QmakeProFileNode::showInSimpleTree() const
 {
-    return hasBuildTargets(projectType());
+    return showInSimpleTree(projectType()) || m_project->rootProjectNode() == this;
 }
 
-bool QmakeProFileNode::hasBuildTargets(QmakeProjectType projectType) const
+ProjectExplorer::FolderNode::AddNewInformation QmakeProFileNode::addNewInformation(const QStringList &files, Node *context) const
+{
+    Q_UNUSED(files)
+    return AddNewInformation(QFileInfo(path()).fileName(), context == this ? 120 : 100);
+}
+
+bool QmakeProFileNode::showInSimpleTree(QmakeProjectType projectType) const
 {
     return (projectType == ApplicationTemplate || projectType == LibraryTemplate);
 }
@@ -1719,9 +1754,9 @@ void QmakeProFileNode::applyEvaluate(EvalResult evalResult, bool async)
                 return;
 
             // delete files && folders && projects
-            removeFileNodes(fileNodes(), this);
+            removeFileNodes(fileNodes());
             removeProjectNodes(subProjectNodes());
-            removeFolderNodes(subFolderNodes(), this);
+            removeFolderNodes(subFolderNodes());
 
             // change project type
             QmakeProjectType oldType = m_projectType;
@@ -1750,19 +1785,19 @@ void QmakeProFileNode::applyEvaluate(EvalResult evalResult, bool async)
             }
         }
 
-        removeFileNodes(fileNodes(), this);
+        removeFileNodes(fileNodes());
         removeProjectNodes(subProjectNodes());
-        removeFolderNodes(subFolderNodes(), this);
+        removeFolderNodes(subFolderNodes());
 
-        bool changesHasBuildTargets = hasBuildTargets() ^ hasBuildTargets(projectType);
+        bool changesShowInSimpleTree = showInSimpleTree() ^ showInSimpleTree(projectType);
 
-        if (changesHasBuildTargets)
-            aboutToChangeHasBuildTargets();
+        if (changesShowInSimpleTree)
+            aboutToChangeShowInSimpleTree();
 
         m_projectType = projectType;
 
-        if (changesHasBuildTargets)
-            hasBuildTargetsChanged();
+        if (changesShowInSimpleTree)
+            showInSimpleTreeChanged();
 
         // really emit here? or at the end? Nobody is connected to this signal at the moment
         // so we kind of can ignore that question for now
@@ -1811,10 +1846,11 @@ void QmakeProFileNode::applyEvaluate(EvalResult evalResult, bool async)
         }
     }
 
+    SortByPath sortByPath;
     qSort(existingProjectNodes.begin(), existingProjectNodes.end(),
-          sortNodesByPath);
-    qSort(newProjectFilesExact);
-    qSort(newProjectFilesCumlative);
+          sortByPath);
+    qSort(newProjectFilesExact.begin(), newProjectFilesExact.end(), sortByPath);
+    qSort(newProjectFilesCumlative.begin(), newProjectFilesCumlative.end(), sortByPath);
 
     QList<ProjectNode*> toAdd;
     QList<ProjectNode*> toRemove;
@@ -1953,13 +1989,46 @@ void QmakeProFileNode::applyEvaluate(EvalResult evalResult, bool async)
 
     m_validParse = (evalResult == EvalOk);
     if (m_validParse) {
+        // create build_pass reader
+        QtSupport::ProFileReader *readerBuildPass = 0;
+        QStringList builds = m_readerExact->values(QLatin1String("BUILDS"));
+        if (builds.isEmpty()) {
+            readerBuildPass = m_readerExact;
+        } else {
+            QString build = builds.first();
+            QHash<QString, QStringList> basevars;
+            QStringList basecfgs = m_readerExact->values(build + QLatin1String(".CONFIG"));
+            basecfgs += build;
+            basecfgs += QLatin1String("build_pass");
+            basevars[QLatin1String("BUILD_PASS")] = QStringList(build);
+            QStringList buildname = m_readerExact->values(build + QLatin1String(".name"));
+            basevars[QLatin1String("BUILD_NAME")] = (buildname.isEmpty() ? QStringList(build) : buildname);
+
+            readerBuildPass = m_project->createProFileReader(this);
+            readerBuildPass->setExtraVars(basevars);
+            readerBuildPass->setExtraConfigs(basecfgs);
+
+            EvalResult evalResult = EvalOk;
+            if (ProFile *pro = readerBuildPass->parsedProFile(m_projectFilePath)) {
+                if (!readerBuildPass->accept(pro, QMakeEvaluator::LoadAll))
+                    evalResult = EvalPartial;
+                pro->deref();
+            } else {
+                evalResult = EvalFail;
+            }
+
+            if (evalResult != EvalOk) {
+                m_project->destroyProFileReader(readerBuildPass);
+                readerBuildPass = 0;
+            }
+        }
 
         // update TargetInformation
-        m_qmakeTargetInformation = targetInformation(m_readerExact);
+        m_qmakeTargetInformation = targetInformation(m_readerExact, readerBuildPass);
         m_resolvedMkspecPath = m_readerExact->resolvedMkSpec();
 
         m_subProjectsNotToDeploy = subProjectsNotToDeploy;
-        setupInstallsList(m_readerExact);
+        setupInstallsList(readerBuildPass);
 
         QString buildDirectory = buildDir();
         // update other variables
@@ -2019,6 +2088,9 @@ void QmakeProFileNode::applyEvaluate(EvalResult evalResult, bool async)
             }
         }
 
+        if (readerBuildPass && readerBuildPass != m_readerExact)
+            m_project->destroyProFileReader(readerBuildPass);
+
         if (m_varValues != newVarValues) {
             QmakeVariablesHash oldValues = m_varValues;
             m_varValues = newVarValues;
@@ -2163,61 +2235,31 @@ QStringList QmakeProFileNode::subDirsPaths(QtSupport::ProFileReader *reader, QSt
     return subProjectPaths;
 }
 
-TargetInformation QmakeProFileNode::targetInformation(QtSupport::ProFileReader *reader) const
+TargetInformation QmakeProFileNode::targetInformation(QtSupport::ProFileReader *reader, QtSupport::ProFileReader *readerBuildPass) const
 {
     TargetInformation result;
-    if (!reader)
+    if (!reader || !readerBuildPass)
         return result;
 
-    QtSupport::ProFileReader *readerBP = 0;
     QStringList builds = reader->values(QLatin1String("BUILDS"));
     if (!builds.isEmpty()) {
         QString build = builds.first();
         result.buildTarget = reader->value(build + QLatin1String(".target"));
-
-        QHash<QString, QStringList> basevars;
-        QStringList basecfgs = reader->values(build + QLatin1String(".CONFIG"));
-        basecfgs += build;
-        basecfgs += QLatin1String("build_pass");
-        basevars[QLatin1String("BUILD_PASS")] = QStringList(build);
-        QStringList buildname = reader->values(build + QLatin1String(".name"));
-        basevars[QLatin1String("BUILD_NAME")] = (buildname.isEmpty() ? QStringList(build) : buildname);
-
-        readerBP = m_project->createProFileReader(this);
-        readerBP->setExtraVars(basevars);
-        readerBP->setExtraConfigs(basecfgs);
-
-        EvalResult evalResult = EvalOk;
-        if (ProFile *pro = readerBP->parsedProFile(m_projectFilePath)) {
-            if (!readerBP->accept(pro, QMakeEvaluator::LoadAll))
-                evalResult = EvalPartial;
-            pro->deref();
-        } else {
-            evalResult = EvalFail;
-        }
-
-        if (evalResult != EvalOk)
-            return result;
-
-        reader = readerBP;
     }
 
     // BUILD DIR
     result.buildDir = buildDir();
 
-    if (reader->contains(QLatin1String("DESTDIR")))
-        result.destDir = reader->value(QLatin1String("DESTDIR"));
+    if (readerBuildPass->contains(QLatin1String("DESTDIR")))
+        result.destDir = readerBuildPass->value(QLatin1String("DESTDIR"));
 
     // Target
-    result.target = reader->value(QLatin1String("TARGET"));
+    result.target = readerBuildPass->value(QLatin1String("TARGET"));
     if (result.target.isEmpty())
         result.target = QFileInfo(m_projectFilePath).baseName();
 
     result.valid = true;
 
-    if (readerBP)
-        m_project->destroyProFileReader(readerBP);
-
     return result;
 }
 
