@@ -263,24 +263,33 @@ def getEditorForFileSuffix(curFile):
     glslEditorSuffixes= ["frag", "vert", "fsh", "vsh", "glsl", "shader", "gsh"]
     pytEditorSuffixes = ["py", "pyw", "wsgi"]
     suffix = __getFileSuffix__(curFile)
-    if suffix in cppEditorSuffixes:
-        editor = waitForObject(":Qt Creator_CppEditor::Internal::CPPEditorWidget")
-    elif suffix in qmlEditorSuffixes:
-        editor = waitForObject(":Qt Creator_QmlJSEditor::QmlJSTextEditorWidget")
-    elif suffix in proEditorSuffixes:
-        editor = waitForObject(":Qt Creator_ProFileEditorWidget")
-    elif suffix in glslEditorSuffixes:
-        editor = waitForObject("{type='GLSLEditor::Internal::GLSLTextEditorWidget' unnamed='1' "
-                               "visible='1' window=':Qt Creator_Core::Internal::MainWindow'}")
-    elif suffix in pytEditorSuffixes:
-        editor = waitForObject(":Qt Creator_PythonEditor::EditorWidget")
-    else:
-        test.log("Trying PlainTextEditor (file suffix: %s)" % suffix)
-        try:
-            editor = waitForObject(":Qt Creator_TextEditor::PlainTextEditorWidget", 3000)
-        except:
-            test.fatal("Unsupported file suffix for file '%s'" % curFile)
-            editor = None
+    try:
+        if suffix in cppEditorSuffixes:
+            editor = waitForObject(":Qt Creator_CppEditor::Internal::CPPEditorWidget")
+        elif suffix in qmlEditorSuffixes:
+            editor = waitForObject(":Qt Creator_QmlJSEditor::QmlJSTextEditorWidget")
+        elif suffix in proEditorSuffixes:
+            editor = waitForObject(":Qt Creator_ProFileEditorWidget")
+        elif suffix in glslEditorSuffixes:
+            editor = waitForObject("{type='GLSLEditor::Internal::GLSLTextEditorWidget' unnamed='1' "
+                                   "visible='1' window=':Qt Creator_Core::Internal::MainWindow'}")
+        elif suffix in pytEditorSuffixes:
+            editor = waitForObject(":Qt Creator_PythonEditor::EditorWidget")
+        else:
+            test.log("Trying PlainTextEditor (file suffix: %s)" % suffix)
+            try:
+                editor = waitForObject(":Qt Creator_TextEditor::PlainTextEditorWidget", 3000)
+            except:
+                test.fatal("Unsupported file suffix for file '%s'" % curFile)
+                editor = None
+    except:
+        f = str(waitForObject(":Qt Creator_Core::Internal::MainWindow").windowTitle).split(" ", 1)[0]
+        if os.path.basename(curFile) == f:
+            test.fatal("Could not find editor although expected file matches.")
+        else:
+            test.fatal("Expected (%s) and current file (%s) do not match. Failed to get editor"
+                       % (os.path.basename(curFile), f))
+        editor = None
     return editor
 
 # helper that determines the file suffix of the given fileName
@@ -299,8 +308,7 @@ def maskSpecialCharsForSearchResult(filename):
 def validateSearchResult(expectedCount):
     searchResult = waitForObject(":Qt Creator_SearchResult_Core::Internal::OutputPaneToggleButton")
     ensureChecked(searchResult)
-    resultTreeView = waitForObject("{type='Find::Internal::SearchResultTreeView' unnamed='1' "
-                                   "visible='1' window=':Qt Creator_Core::Internal::MainWindow'}")
+    resultTreeView = waitForObject(":Qt Creator_Find::Internal::SearchResultTreeView")
     counterLabel = waitForObject("{type='QLabel' unnamed='1' visible='1' text?='*matches found.' "
                                  "window=':Qt Creator_Core::Internal::MainWindow'}")
     matches = cast((str(counterLabel.text)).split(" ", 1)[0], "int")
@@ -316,7 +324,9 @@ def validateSearchResult(expectedCount):
             rect = resultTreeView.visualRect(chIndex)
             doubleClick(resultTreeView, rect.x+5, rect.y+5, 0, Qt.LeftButton)
             editor = getEditorForFileSuffix(itemText)
-            waitFor("lineUnderCursor(editor) == text", 2000)
+            if not waitFor("lineUnderCursor(editor) == text", 2000):
+                test.warning("Jumping to search result '%s' is pretty slow." % text)
+                waitFor("lineUnderCursor(editor) == text", 2000)
             test.compare(lineUnderCursor(editor), text)
 
 # this function invokes context menu and command from it
@@ -367,7 +377,8 @@ def openDocument(treeElement):
             item = waitForObjectItem(navigator, treeElement)
         doubleClickItem(navigator, treeElement, 5, 5, 0, Qt.LeftButton)
         mainWindow = waitForObject(":Qt Creator_Core::Internal::MainWindow")
-        waitFor("item.text in str(mainWindow.windowTitle)")
+        expected = str(item.text).split("/")[-1]
+        waitFor("expected in str(mainWindow.windowTitle)")
         return True
     except:
         return False
